/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.syncclient.javagui;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import com.funambol.foundation.pdi.event.Calendar;

/**
 * The calendar modification panel.
 *
 * @version $Id: CalendarModify.java,v 1.3 2006/09/07 09:03:05 luigia Exp $
 */
public class CalendarModify extends Panel
implements ActionListener, ConfigurationParameters {

    //---------------------------------------------------------- Private data

    //
    // The window containing this panel
    //
    private MainWindow          mw      = null          ;
    private CalendarForm        form    = null          ;
    private CheckCalendarFields cfields = null          ;
    private Language            ln      = new Language();
    private Calendar            cal     = new Calendar();

    //---------------------------------------------------------- Public methods

    /**
     * Creates the panel.
     *
     * @param mw the window containing this panel
     */
    public CalendarModify(MainWindow mw) {

        Button          butOk        = null ;
        Button          butCancel    = null ;
        Panel           buttonPanel  = null ;

        Label           title        = null ;

        this.mw = mw;

        setLayout(new BorderLayout());
        title = new Label(ln.getString ("calendar_modify") );

        form = new CalendarForm(mw);

        butOk = new Button      (ln.getString ("ok")         ) ;
        butOk.setActionCommand  ("ok"                        ) ;
        butOk.addActionListener (this                        ) ;

        butCancel = new Button      (ln.getString ("cancel") ) ;
        butCancel.setActionCommand  ("cancel"                ) ;
        butCancel.addActionListener (this                    ) ;

        buttonPanel = new Panel();
        buttonPanel.setLayout(new GridLayout(1,2));
        buttonPanel.add (butOk     ) ;
        buttonPanel.add (butCancel ) ;

        add(title       , BorderLayout.NORTH  ) ;
        add(form        , BorderLayout.CENTER ) ;
        add(buttonPanel , BorderLayout.SOUTH  ) ;
    }

    /**
     * Invoked when an action occurs (i.e. a button is pressed).
     *
     * @param evt the occurred action
     */
    public void actionPerformed(ActionEvent evt) {

        if (evt.getActionCommand().equals("ok")) {
            cfields = new CheckCalendarFields(mw, form);
            boolean isAllRight = false;

            isAllRight = cfields.areFieldsRight();

            if (!isAllRight) {
                cfields.show();
                mw.setEnabled(false);
            } else {
                saveCalendar();
            }

        } else if (evt.getActionCommand().equals("cancel")) {
            mw.show(KEY_CALENDARLIST);
        }
    }

    //---------------------------------------------------------- Protected methods

    /**
     * Sets the fields with the values contained
     * in the specified Calendar object
     * and the checkbox with the specified type.
     *
     * @param calendar the Calendar object which is the source of the values
     */
    protected void fillFields(Calendar calendar) {
        form.setFields(calendar);
        cal = calendar;
    }

    //---------------------------------------------------------- Private methods

    /**
     * Saves the modified calendar.
     */
    private void saveCalendar() {
        mw.writeModCalendar(form.getFields(), mw.getCurrentIndex().intValue());
        mw.show(KEY_CALENDARLIST);
    }

}
